#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "msph/err.h"
#include "msph/token.h"

#define PRINTERR(ctx) \
	do { \
		fprintf(stderr, "msph_err: %x (%d)\n", (ctx).err, \
		    (ctx).err_info); \
	} while (0)

struct msph_opts {
	int		tokenize_only;
	const char	*in_path;
	const char	*out_path;
} run_opts = {
	0,
	NULL,
	NULL
};


#define USAGE \
	"msph: micro sappho\n" \
	"\tUsage: msph [-t] [-o out_path] [in_path]\n" \
	"\n" \
	"\t-t	- print tokenization\n" \
	"\t-o	- output path\n"


void
print_usage(void)
{
	printf("%s", USAGE);
}

int run(struct msph_opts *);

int
main(int argc, char *argv[])
{
	int opt;

	while ((opt = getopt(argc, argv, "to:")) != -1) {
		switch (opt) {
		case 't':
			run_opts.tokenize_only = 1;
			break;
		case 'o':
			run_opts.out_path = optarg;
			break;
		case '?':
		default:
			print_usage();
			exit(EXIT_FAILURE);
		}
	}

	argc -= optind;
	argv += optind;

	if (argc > 1) {
		print_usage();
		exit(EXIT_FAILURE);
	}

	if (argc == 1)
		run_opts.in_path = argv[0];

	if (run(&run_opts))
		exit(EXIT_FAILURE);
}


int
run(struct msph_opts *opts)
{
	int ret;
	struct msph_ctx ctx;
	struct msph_token_stream *s;
	FILE *in, *out;

	in = out = NULL;

	msph_ctx_init(&ctx);

	if (opts->in_path == NULL) {
		in = stdin; opts->in_path = "stdin";
	} else {
		if ((in = fopen(opts->in_path, "r")) == NULL) {
			MSPH_ERR(&ctx, MSPH_ERR_SYS);
			goto err;
		}
	}

	if (opts->out_path == NULL) {
		out = stdout;
	} else {
		if ((out = fopen(opts->out_path, "w")) == NULL) {
			MSPH_ERR(&ctx, MSPH_ERR_SYS);
			goto err;
		}
	}

	if ((s = msph_token_stream_file(&ctx, in)) == NULL) {
		goto err;
	}

	printf("msph v0, parsing %s\n", opts->in_path);

	if ((ret = msph_token_stream_print(s, out)) == -1)
		goto err;

	if (opts->tokenize_only)
		goto exit;

exit:
	if (msph_token_stream_close(s) == -1)
		goto err;

	return (0);

err:
	PRINTERR(ctx);

	if (in)
		fclose(in);
	if (out)
		fclose(out);
	return (-1);
}
